@@ -223,7 +223,9 @@ class ContinuousGenerator(keras.utils.Sequence):
         if batch_function is None:
             batch_function = lambda image: image
         if label_function is None:
-            raise ValueError("If batch_function is not None, then label_function cannot be None. Please define a valid label_function.")
+            raise ValueError(
+                "If batch_function is not None, then label_function cannot be None. Please define a valid label_function."
+            )
         if min_data_size is None:
             min_data_size = min(batch_size * 10, max_data_size - 1)
 
@@ -308,6 +310,10 @@ class ContinuousGenerator(keras.utils.Sequence):
             )
             time.sleep(0.1)
             current_data = list(self.data)
+            print(
+                "                                                                        ",
+                end="\r",
+            )
 
         self.new_epoch = True
         if self.augmentation and isinstance(self.augmentation, Feature):
@@ -332,7 +338,13 @@ class ContinuousGenerator(keras.utils.Sequence):
 
         batch_size = self._batch_size
 
-        subset = self.current_data[idx * batch_size : (idx + 1) * batch_size]
+        try:
+            subset = self.current_data[idx * batch_size : (idx + 1) * batch_size]
+        except IndexError:
+            return self[0]
+
+        if len(subset) < self.batch_size:
+            return self[0]
 
         for d in subset:
             d["usage"] += 1
@@ -343,7 +355,7 @@ class ContinuousGenerator(keras.utils.Sequence):
         return np.array(data), np.array(labels)
 
     def __len__(self):
-        steps = int((len(self.current_data) // self._batch_size))
+        steps = int((self.min_data_size // self._batch_size))
         assert (
             steps > 0
         ), "There needs to be at least batch_size number of datapoints. Try increasing min_data_size."
@@ -401,3 +413,15 @@ class ContinuousGenerator(keras.utils.Sequence):
         else:
             features.update()
             return features.resolve()
+
+
+class PyTorchContinuousGenerator(ContinuousGenerator):
+    """Extends the ContinuousGenerator to support PyTorch models.
+
+    This class is used to generate batches of data for PyTorch models."""
+
+    def __getitem__(self, idx):
+        import torch
+
+        X, y = super().__getitem__(idx)
+        return torch.from_numpy(X).to(torch.float), torch.from_numpy(y).to(torch.float)
\ No newline at end of file
