@@ -18,14 +18,18 @@ Ellipsoid
 """
 
 
+from pint import Quantity
+from . import image
+from deeptrack.backend.units import ConversionTable
 from typing import Callable, Tuple
+
 import numpy as np
 
 from . import backend as D
 from .features import Feature, MERGE_STRATEGY_APPEND
-from .image import Image
-from . import image
+from . import pad_image_to_fft, Image
 from .types import PropertyLike, ArrayLike
+from . import units as u
 import warnings
 
 
@@ -69,6 +73,9 @@ class Scatterer(Feature):
 
     __list_merge_strategy__ = MERGE_STRATEGY_APPEND
     __distributed__ = False
+    __conversion_table__ = ConversionTable(
+        position=(u.pixel, u.pixel), z=(u.pixel, u.pixel), voxel_size=(u.meter, u.meter)
+    )
 
     def __init__(
         self,
@@ -77,6 +84,8 @@ class Scatterer(Feature):
         value: PropertyLike[float] = 1.0,
         position_unit: PropertyLike[str] = "pixel",
         upsample: PropertyLike[int] = 1,
+        voxel_size=None,
+        pixel_size=None,
         **kwargs
     ):
         self._processed_properties = False
@@ -86,25 +95,15 @@ class Scatterer(Feature):
             value=value,
             position_unit=position_unit,
             upsample=upsample,
-            **kwargs
+            voxel_size=voxel_size,
+            pixel_size=pixel_size,
+            **kwargs,
         )
 
     def _process_properties(self, properties: dict) -> dict:
         # Rescales the position property
+        properties = super()._process_properties(properties)
         self._processed_properties = True
-        if "position" in properties:
-            if properties["position_unit"] == "meter":
-                properties["position"] = (
-                    np.array(properties["position"])
-                    / np.array(properties["voxel_size"])[: len(properties["position"])]
-                    / properties.get("upscale", 1)
-                )
-                properties["z"] = (
-                    np.array(properties["z"])
-                    / np.array(properties["voxel_size"])[: len(properties["position"])]
-                    / properties.get("upscale", 1)
-                )
-
         return properties
 
     def _process_and_get(
@@ -230,6 +229,11 @@ class Ellipse(Scatterer):
         Upsamples the calculations of the pixel occupancy fraction.
     """
 
+    __conversion_table__ = ConversionTable(
+        radius=(u.meter, u.pixel),
+        rotation=(u.radian, u.radian),
+    )
+
     def __init__(
         self,
         radius: PropertyLike[float] = 1e-6,
@@ -264,7 +268,7 @@ class Ellipse(Scatterer):
     def get(self, *ignore, radius, rotation, voxel_size, **kwargs):
 
         # Create a grid to calculate on
-        rad = radius[:2] / voxel_size[:2]
+        rad = radius[:2]
         ceil = int(np.max(np.ceil(rad)))
         X, Y = np.meshgrid(np.arange(-ceil, ceil), np.arange(-ceil, ceil))
 
@@ -303,13 +307,17 @@ class Sphere(Scatterer):
         Upsamples the calculations of the pixel occupancy fraction.
     """
 
+    __conversion_table__ = ConversionTable(
+        radius=(u.meter, u.pixel),
+    )
+
     def __init__(self, radius: PropertyLike[float] = 1e-6, **kwargs):
         super().__init__(radius=radius, **kwargs)
 
     def get(self, image, radius, voxel_size, **kwargs):
 
         # Create a grid to calculate on
-        rad = radius / voxel_size
+        rad = radius * np.ones(3)
         rad_ceil = np.ceil(rad)
         x = np.arange(-rad_ceil[0], rad_ceil[0])
         y = np.arange(-rad_ceil[1], rad_ceil[1])
@@ -346,6 +354,11 @@ class Ellipsoid(Scatterer):
         Upsamples the calculations of the pixel occupancy fraction.
     """
 
+    __conversion_table__ = ConversionTable(
+        radius=(u.meter, u.pixel),
+        rotation=(u.radian, u.radian),
+    )
+
     def __init__(
         self,
         radius: PropertyLike[float] = 1e-6,
@@ -403,15 +416,15 @@ class Ellipsoid(Scatterer):
 
     def get(self, image, radius, rotation, voxel_size, **kwargs):
 
-        radius_in_pixels = radius / voxel_size
+        radius_in_pixels = radius
 
-        max_rad = np.max(radius) / voxel_size
+        max_rad = np.max(radius)
         rad_ceil = np.ceil(max_rad)
 
         # Create grid to calculate on
-        x = np.arange(-rad_ceil[0], rad_ceil[0])
-        y = np.arange(-rad_ceil[1], rad_ceil[1])
-        z = np.arange(-rad_ceil[2], rad_ceil[2])
+        x = np.arange(-rad_ceil, rad_ceil)
+        y = np.arange(-rad_ceil, rad_ceil)
+        z = np.arange(-rad_ceil, rad_ceil)
         X, Y, Z = np.meshgrid(x, y, z)
 
         # Rotate the grid
@@ -462,10 +475,14 @@ class MieScatterer(Scatterer):
     collection_angle : "auto" or float
         The maximum collection angle in radians. If "auto", this
         is calculated from the objective NA (which is true if the objective is
-        the limiting
-        aperature).
-    polarization_angle : float
-        Angle of the polarization of the incoming light relative to the x-axis.
+        the limiting aperature).
+    input_polarization: float or Quantity
+        Defines the polarization angle of the input. For simulating circularly
+        polarized light we recommend a coherent sum of two simulated fields. For
+        unpolarized light we recommend a incoherent sum of two simulated fields.
+    output_polarization: float or Quantity or None
+        If None, the output light is not polarized. Otherwise defines the angle of the
+        polarization filter after the sample. For off-axis, keep the same as input_polarization.
     L : int or str
         The number of terms used to evaluate the mie theory. If `"auto"`,
         it determines the number of terms automatically.
@@ -478,15 +495,37 @@ class MieScatterer(Scatterer):
         camera plane. Used if `position` is of length 2.
     """
 
+    __gpu_compatible__ = True
+
+    __conversion_table__ = ConversionTable(
+        radius=(u.meter, u.meter),
+        polarization_angle=(u.radian, u.radian),
+        collection_angle=(u.radian, u.radian),
+        wavelength=(u.meter, u.meter),
+        offset_z=(u.meter, u.meter),
+    )
+
     def __init__(
         self,
         coefficients: Callable[..., Callable[[int], Tuple[ArrayLike, ArrayLike]]],
+        input_polarization=0,
+        output_polarization=0,
         offset_z: PropertyLike[str] = "auto",
-        polarization_angle: PropertyLike[float] = 0,
         collection_angle: PropertyLike[str] = "auto",
         L: PropertyLike[str] = "auto",
+        refractive_index_medium=None,
+        wavelength=None,
+        NA=None,
+        padding=(0,) * 4,
+        output_region=None,
+        polarization_angle=None,
         **kwargs
     ):
+        if polarization_angle is not None:
+            warnings.warn(
+                "polarization_angle is deprecated. Please use input_polarization instead"
+            )
+            input_polarization = polarization_angle
         kwargs.pop("is_field", None)
         kwargs.pop("crop_empty", None)
 
@@ -495,10 +534,17 @@ class MieScatterer(Scatterer):
             crop_empty=False,
             L=L,
             offset_z=offset_z,
-            polarization_angle=polarization_angle,
+            input_polarization=input_polarization,
+            output_polarization=output_polarization,
             collection_angle=collection_angle,
             coefficients=coefficients,
-            **kwargs
+            refractive_index_medium=refractive_index_medium,
+            wavelength=wavelength,
+            NA=NA,
+            padding=padding,
+            output_region=output_region,
+            polarization_angle=polarization_angle,
+            **kwargs,
         )
 
     def _process_properties(self, properties):
@@ -508,7 +554,7 @@ class MieScatterer(Scatterer):
         if properties["L"] == "auto":
             try:
                 v = 2 * np.pi * np.max(properties["radius"]) / properties["wavelength"]
-                properties["L"] = int(np.ceil(v + 4 * (v ** (1 / 3)) + 2))
+                properties["L"] = int(np.floor((v + 4 * (v ** (1 / 3)) + 1)))
             except (ValueError, TypeError):
                 pass
         if properties["collection_angle"] == "auto":
@@ -518,10 +564,13 @@ class MieScatterer(Scatterer):
 
         if properties["offset_z"] == "auto":
             properties["offset_z"] = (
-                32
+                np.min(
+                    np.array(properties["output_region"][2:])
+                    - properties["output_region"][:2]
+                )
+                / 2
                 * min(properties["voxel_size"][:2])
                 / np.sin(properties["collection_angle"])
-                * properties["upscale"]
             )
         return properties
 
@@ -529,7 +578,7 @@ class MieScatterer(Scatterer):
         self,
         inp,
         position,
-        upscaled_output_region,
+        output_region,
         voxel_size,
         padding,
         wavelength,
@@ -537,38 +586,61 @@ class MieScatterer(Scatterer):
         L,
         offset_z,
         collection_angle,
-        polarization_angle,
+        input_polarization,
+        output_polarization,
         coefficients,
-        upscale=1,
         **kwargs
     ):
 
-        xSize = (
-            padding[2]
-            + upscaled_output_region[2]
-            - upscaled_output_region[0]
-            + padding[0]
-        )
-        ySize = (
-            padding[3]
-            + upscaled_output_region[3]
-            - upscaled_output_region[1]
-            + padding[1]
-        )
-        arr = image.pad_image_to_fft(np.zeros((xSize, ySize)))
+        xSize = padding[2] + output_region[2] - output_region[0] + padding[0]
+        ySize = padding[3] + output_region[3] - output_region[1] + padding[1]
 
+        arr = pad_image_to_fft(np.zeros((xSize, ySize)))
+        position = np.array(position)
+        pos_floor = np.floor(position)
+        pos_digits = position - pos_floor
         # Evluation grid
-        x = np.arange(-padding[0], arr.shape[0] - padding[0]) - (position[0]) * upscale
-        y = np.arange(-padding[1], arr.shape[1] - padding[1]) - (position[1]) * upscale
+        x = (
+            np.arange(-padding[0], arr.shape[0] - padding[0])
+            - arr.shape[0] // 2
+            + padding[0]
+            - pos_digits[0]
+        )
+        y = (
+            np.arange(-padding[1], arr.shape[1] - padding[1])
+            - arr.shape[1] // 2
+            + padding[1]
+            - pos_digits[1]
+        )
+
+        x = np.roll(x, int(-arr.shape[0] // 2 + padding[0] + pos_floor[0]), 0)
+        y = np.roll(y, int(-arr.shape[1] // 2 + padding[1] + pos_floor[1]), 0)
         X, Y = np.meshgrid(x * voxel_size[0], y * voxel_size[1], indexing="ij")
 
+        X = image.maybe_cupy(X)
+        Y = image.maybe_cupy(Y)
+
         R2 = np.sqrt(X ** 2 + Y ** 2)
         R3 = np.sqrt(R2 ** 2 + (offset_z) ** 2)
         ct = offset_z / R3
 
-        ANGLE = np.arctan2(Y, X) + polarization_angle
-        COS2 = np.square(np.cos(ANGLE))
-        SIN2 = 1 - COS2
+        angle = np.arctan2(Y, X)
+        if isinstance(input_polarization, (float, int, Quantity)):
+
+            if isinstance(input_polarization, Quantity):
+                input_polarization = input_polarization.to("rad")
+                input_polarization = input_polarization.magnitude
+
+            S1_coef = np.sin(angle + input_polarization)
+            S2_coef = np.cos(angle + input_polarization)
+
+        if isinstance(output_polarization, (float, int, Quantity)):
+            if isinstance(input_polarization, Quantity):
+                output_polarization = output_polarization.to("rad")
+                output_polarization = output_polarization.magnitude
+
+            S1_coef *= np.sin(angle + output_polarization)
+            S2_coef *= np.cos(angle + output_polarization)
 
         ct_max = np.cos(collection_angle)
 
@@ -576,7 +648,6 @@ class MieScatterer(Scatterer):
         k = 2 * np.pi / wavelength * refractive_index_medium
 
         # Harmonics
-
         A, B = coefficients(L)
         PI, TAU = D.mie_harmonics(ct, L)
 
@@ -584,15 +655,15 @@ class MieScatterer(Scatterer):
         E = [(2 * i + 1) / (i * (i + 1)) for i in range(1, L + 1)]
 
         # Scattering terms
-        S1 = sum([E[i] * A[i] * TAU[i] + E[i] * B[i] * PI[i] for i in range(0, L)])
-        S2 = sum([E[i] * B[i] * TAU[i] + E[i] * A[i] * PI[i] for i in range(0, L)])
+        S1 = sum([E[i] * A[i] * PI[i] + E[i] * B[i] * TAU[i] for i in range(0, L)])
+        S2 = sum([E[i] * B[i] * PI[i] + E[i] * A[i] * TAU[i] for i in range(0, L)])
 
         field = (
             (ct > ct_max)
             * 1j
             / (k * R3)
             * np.exp(1j * k * (R3 - offset_z))
-            * (S1 * COS2 + S2 * SIN2)
+            * (S2 * S2_coef + S1 * S1_coef)
         )
 
         return np.expand_dims(field, axis=-1)
@@ -633,8 +704,13 @@ class MieSphere(MieScatterer):
         The maximum collection angle in radians. If "auto", this
         is calculated from the objective NA (which is true if the objective
         is the limiting aperature).
-    polarization_angle : float
-        Angle of the polarization of the incoming light relative to the x-axis.
+    input_polarization: float or Quantity
+        Defines the polarization angle of the input. For simulating circularly
+        polarized light we recommend a coherent sum of two simulated fields. For
+        unpolarized light we recommend a incoherent sum of two simulated fields.
+    output_polarization: float or Quantity or None
+        If None, the output light is not polarized. Otherwise defines the angle of the
+        polarization filter after the sample. For off-axis, keep the same as input_polarization.
     """
 
     def __init__(
@@ -657,7 +733,7 @@ class MieSphere(MieScatterer):
             coefficients=coeffs,
             radius=radius,
             refractive_index=refractive_index,
-            **kwargs
+            **kwargs,
         )
 
 
@@ -698,8 +774,13 @@ class MieStratifiedSphere(MieScatterer):
         The maximum collection angle in radians. If "auto", this
         is calculated from the objective NA (which is true if the objective
         is the limiting aperature).
-    polarization_angle : float
-        Angle of the polarization of the incoming light relative to the x-axis.
+    input_polarization: float or Quantity
+        Defines the polarization angle of the input. For simulating circularly
+        polarized light we recommend a coherent sum of two simulated fields. For
+        unpolarized light we recommend a incoherent sum of two simulated fields.
+    output_polarization: float or Quantity or None
+        If None, the output light is not polarized. Otherwise defines the angle of the
+        polarization filter after the sample. For off-axis, keep the same as input_polarization.
     """
 
     def __init__(
@@ -726,5 +807,5 @@ class MieStratifiedSphere(MieScatterer):
             coefficients=coeffs,
             radius=radius,
             refractive_index=refractive_index,
-            **kwargs
+            **kwargs,
         )
